/********************************************************************************************************
 * @file	string.c
 *
 * @brief	This is the source file for TLSR8232
 *
 * @author	Driver Group
 * @date	May 8, 2018
 *
 * @par     Copyright (c) 2018, Telink Semiconductor (Shanghai) Co., Ltd. ("TELINK")
 *          All rights reserved.
 *
 *          Redistribution and use in source and binary forms, with or without
 *          modification, are permitted provided that the following conditions are met:
 *
 *              1. Redistributions of source code must retain the above copyright
 *              notice, this list of conditions and the following disclaimer.
 *
 *              2. Unless for usage inside a TELINK integrated circuit, redistributions
 *              in binary form must reproduce the above copyright notice, this list of
 *              conditions and the following disclaimer in the documentation and/or other
 *              materials provided with the distribution.
 *
 *              3. Neither the name of TELINK, nor the names of its contributors may be
 *              used to endorse or promote products derived from this software without
 *              specific prior written permission.
 *
 *              4. This software, with or without modification, must only be used with a
 *              TELINK integrated circuit. All other usages are subject to written permission
 *              from TELINK and different commercial license may apply.
 *
 *              5. Licensee shall be solely responsible for any claim to the extent arising out of or
 *              relating to such deletion(s), modification(s) or alteration(s).
 *
 *          THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 *          ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *          WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 *          DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDER BE LIABLE FOR ANY
 *          DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *          (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *          LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *          ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *          (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *          SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *******************************************************************************************************/
#include "string.h"
#include "assert.h"
#include "types.h"

char* strcpy(char * dst0, const char * src0) 
{
    char *s = dst0;
    while ((*dst0++ = *src0++));
    return s;
}

char * strchr(const char *s, int c) 
{
    do {
        if (*s == c) {
            return (char*) s;
        }
    } while (*s++);
    return (0);
}

int memcmp(const void * m1, const void *m2, unsigned int n) 
{
	unsigned char *s1 = (unsigned char *) m1;
	unsigned char *s2 = (unsigned char *) m2;

	while (n--) {
		if (*s1 != *s2) {
			return *s1 - *s2;
		}
		s1++;
		s2++;
	}
	return 0;
}

void * memchr(register const void * src_void, int c, unsigned int length) 
{
	const unsigned char *src = (const unsigned char *) src_void;

	while (length-- > 0) {
		if (*src == c)
			return (void *) src;
		src++;
	}
	return NULL;
}

void * memmove(void * dest, const void * src, unsigned int n) 
{
    char * d = (char *)dest;
    char * s = (char *)src;

    while (n--)
        *d++ = *s++;

    return dest;
}

void bcopy(register char * src, register char * dest, int len) 
{
    if (dest < src)
        while (len--)
            *dest++ = *src++;
    else {
        char *lasts = src + (len - 1);
        char *lastd = dest + (len - 1);
        while (len--)
            *(char *) lastd-- = *(char *) lasts--;
    }
}

#if (__DEBUG__ )
u8 *T_bufP;
u8 T_leng;
#endif
void * memset(void * dest, int val, unsigned int len) 
{
    if ( dest == NULL) {
        while(1);
    }
    register unsigned char *ptr = (unsigned char*) dest;
    while (len-- > 0)
        *ptr++ = (unsigned char)val;
    return dest;
}

void* memcpy(void * out, const void * in, unsigned int length) 
{
    if ( length == 0 ) {
        return NULL;
    }

    if ( out == NULL || in == NULL) {
        while(1);
    }
    bcopy((char *) in, (char *) out, (int) length);
    return out;
}

// for performance, assume lenght % 4 == 0,  and no memory overlapped
void memcpy4(void * d, const void * s, unsigned int length)
{
    int* dst = (int*)d;
    int* src = (int*)s;
    assert((((int)dst) >> 2) << 2 == ((int)dst));            // address must alighn to 4
    assert((((int)src) >> 2) << 2 == ((int)src));            // address must alighn to 4
    assert((length >> 2) << 2 == length);                    // lenght % 4 == 0
    assert(( ((char*)dst) + length <= (const char*)src) || (((const char*)src) + length <= (char*)dst));    //  no overlapped
    unsigned int len = length >> 2;
    while (len --) {
        *dst++ = *src++;
    }
}

unsigned int strlen(const char *str) 
{
    unsigned int len = 0;

    if (str != NULL) {
        while (*str++) {

            len++;

        }
    }

    return len;
}

int strcmp(const char* firstString, const char* secondString) 
{
    while (*firstString == *secondString) {
        if (*firstString == '\0') {
            return 0;
        }
        ++firstString;
        ++secondString;
    }
    if (((unsigned char) *firstString - (unsigned char) *secondString) < 0) {
        return -1;
    }
    return 1;
}

char * strncpy(char *s, const char *t, unsigned int n) 
{
    char *p = s;
    unsigned int i = 0;

    if (!s)
        return s;

    while (t && i < n) {
        *s++ = *t++;
        i++;
    }

    if (!t) {
        do
            *s++ = '\0';
        while (i++ < n);
    }
    return p;
}

int ismemzero4(void *data, unsigned int len)
{
    int *p = (int*)data;
    len = len >> 2;
    for (int i = 0; i < len; ++i) {
        if (*p) {
            return 0;
        }
        ++p;
    }
    return 1;
}

int ismemf4(void *data, unsigned int len)
{
    int *p = (int*)data;
    len = len >> 2;
    for (int i = 0; i < len; ++i) {
        if (*p != 0xffffffff) {
            return 0;
        }
        ++p;
    }
    return 1;
}

void * memset4(void * dest, int val, unsigned int len) 
{
    int *p = (int*)dest;
    len = len >> 2;
    for(int i = 0; i < len; ++i){
        *p++ = val;
    }
    return dest;
}

void zeromem4(void *data, unsigned int len)
{
    memset4(data, 0, len);
}


